/*
 * LCD.c
 *
 *  Created on: 25.07.2012
 *      Author: Gerrit
 */

#include "main.h"
#include "lcd.h"

unsigned char ucLCD_x, ucLCD_y, ucLCD_maxx;
unsigned int uiBase_y[4] = {0x80, 0xc0, 0x94, 0xd4};             // example for 4x20 display
          // 1. line 0x00..0x13 + Bit7==0x80
          // 2. line 0x40..0x53 + Bit7==0xc0
          // 3. line 0x14..0x27 + Bit7==0x94
          // 4. line 0x54..0x67 + Bit7==0xd4
          // BIT7 nessesary for command "Set DD RAM address"


//********************************************************************************
// Description  : Waiting for Busy-flag
// Input        : none
// Output       : none
//********************************************************************************
void vLCD_delay(void) {
  volatile unsigned int i;

  for (i=2; i>0; i--);                          // delay loop, 78 loops are abt 50us @16MHz clock
}

//********************************************************************************
//Description  : Sends byte to lcd in 4bit-mode
//Input        : data - data to send
//             : reg - specifies the lcd-register
//Output       : none
//********************************************************************************
void vSendByteToLCD_4BIT(unsigned char ucData, unsigned char ucReg) {
// Data valid at HIGH-LOW transition of enable signals !!
// High nibble (BIT7-BIT4) are transfered first
   if (ucReg == COMMAND)
      SELECT_COMMAND_REG;
   else
      SELECT_DATA_REG;

   // high nibble ber BIT7..BIT4 senden
   DISPLAY_ENABLE_HIGH;
   LCD_DATA_OUT = ucData>>4;
   vLCD_delay();
   DISPLAY_ENABLE_LOW;
   vLCD_delay();

   // low nibble ber BIT7..BIT4 senden
   DISPLAY_ENABLE_HIGH;
   LCD_DATA_OUT = ucData;
   vLCD_delay();
   DISPLAY_ENABLE_LOW;
   vLCD_delay();
}

// ********************************************************************************
// Description  : Sends byte to the LCD's instruction-register in 8bit-mode
// Input        : data - data to send
// Output       : none
// ********************************************************************************
void vSendByteToLCD_8BIT(unsigned char ucData) {
// Data valid at HIGH-LOW transition of enable signals !!
// for this function only control-registers are written
   SELECT_COMMAND_REG;
   DISPLAY_ENABLE_HIGH;
//   LCD_DATA_OUT |= (ucData & 0xf0);
   LCD_DATA_OUT = ucData>>4;
   vLCD_delay();
   DISPLAY_ENABLE_LOW;
   vLCD_delay();
}

//********************************************************************************
//Description  : initialize the LCD controller
//             : needs about 15ms to start
//Input        : lcd_columns	- count of columns
//Output       : none
//********************************************************************************
void vLCD_init(unsigned char ucLCDcolumns) {
//wait for more than 40ms after Vcc rises to 2.7V
//wait for more than 15ms after Vcc rises to 4.5V
  volatile unsigned int i;

  ucLCD_maxx = ucLCDcolumns;
  LCD_DATA_DIR |= DATA_LINE_BITMASK;                           //sets data lines as outputs
  LCD_COMM_DIR |= CONTROL_LINE_BITMASK;                        //sets control lines as outputs
  SELECT_COMMAND_REG;
  DISPLAY_ENABLE_LOW;

  for (i=500; i>0; i--)                                        //300x50us=15ms
    vLCD_delay();

  vSendByteToLCD_8BIT(0x30);                                   //function set (Interface 8bit)
  for (i=82; i>0; i--)                                         //82x50us=4.1ms
    vLCD_delay();
  vSendByteToLCD_8BIT(0x30);                                   //function set (Interface 8bit)
  for (i=3; i>0; i--)                                          //3x50us=150us
    vLCD_delay();
  vSendByteToLCD_8BIT(0x30);                                   //function set (Interface 8bit)
  vLCD_delay();
  vSendByteToLCD_8BIT(0x20);                                   //function set 4bit (Interface 8bit)
  vLCD_delay();
  vSendByteToLCD_4BIT(0x28,COMMAND);                           //function set + number of display lines + character font
  vLCD_delay();

  vLCD_clear();												   //clear display
  vSendByteToLCD_4BIT(0x06,COMMAND);                           //entry mode set, Cursor moves, no Display shift
}

//********************************************************************************
// Description  : set the LCD display position x=0..39 y=0..3
//              :
// Input        : x - X-position
//              : y - Y-position
// Output       : none
//********************************************************************************
void vLCD_gotoxy(unsigned char ucX, unsigned char ucY) {
   if (ucY > 3) ucY=0;
   vSendByteToLCD_4BIT(uiBase_y[ucY]+ucX, COMMAND);
   ucLCD_x=ucX;
   ucLCD_y=ucY;
}

//********************************************************************************
// Description  : clears the LCD
// Input        : none
// Output       : none
//********************************************************************************
void vLCD_clear(void) {
   volatile unsigned int i;

   vSendByteToLCD_4BIT(0x0c, COMMAND);	                          //Display ON, Cursor and Blinking off
   vLCD_delay();
   vSendByteToLCD_4BIT(0x01, COMMAND);	                          //function display clear
   ucLCD_x=ucLCD_y=0;											  //Cursor is at 0,0 then

   for (i=40; i>0; i--)                                           //40x50us=2ms, wait for clear operation (>=1.52ms)
    vLCD_delay();
}

//********************************************************************************
// Description  : write a char to the LCD
// Input        : c - character
// Output       : none
//********************************************************************************
void vLCD_putchar(unsigned char ucC) {
   if (ucLCD_x >= ucLCD_maxx) {
      ucLCD_y++;
      vLCD_gotoxy(0,ucLCD_y);
   }
   vLCD_delay();

   vSendByteToLCD_4BIT(ucC, DATA);
   ucLCD_x++;
}

//********************************************************************************
// Description  : write the string str to the LCD
// Input        : *str	pointer to string
// Output       : none
//********************************************************************************
void vLCD_puts(char *ucStr) {
   while(*ucStr!=0x0)
      vLCD_putchar(*ucStr++);
}

//********************************************************************************
// Description  : write an unsigned integer to LCD
// Input        : unsigned integer uiX
// Output       : none
//********************************************************************************
void vLCD_putui(unsigned int uiX) {
  	unsigned int y, digit=0;
	y=uiX/10000;
	if(y) {
    	vLCD_putchar(y+0x30);
    	digit += 1;
    	uiX -= (y*10000);
	}
    y=uiX/1000;
    if(y || digit) {
    	vLCD_putchar(y+0x30);
    	digit += 1;
    	uiX -= (y*1000);
    }
    y=uiX/100;
    if(y || digit) {
    	vLCD_putchar(y+0x30);
    	digit += 1;
    	uiX -= (y*100);
    }
    y=uiX/10;
    if(y || digit) {
    	vLCD_putchar(y+0x30);
    	digit += 1;
    	uiX -= (y*10);
    }
    vLCD_putchar(uiX+0x30);
//    for (y=digit; y<4; y++ )
//    	vLCD_putchar(' ');		//fill up with spaces
}

//********************************************************************************
// Description  : write an signed integer to LCD
// Input        : signed integer iX
// Output       : none
//********************************************************************************
void vLCD_puti(int iX) {
  	if (iX < 0)	{
		vLCD_putchar('-');
		iX = ~iX + 1;
	}
	vLCD_putui((unsigned int)iX);
}

// Ende LCD.c



