/*
 * MSP430_SWI2C_Master.c
 */

//******************************************************************************
//  MSP430F20xx - I2C Master Transmitter and Receiver via CPU Software (GPIO)
//
//  Description: This code library configures the MSP430 as an I2C master device
//  capable of transmitting and receiving bytes using GPIO pins.  Specific I/O
//  pins can be selected in the corresponding header file.  By default, the same
//  pins that are used by the USI module are selected.
//
//                  Master
//                  MSP430
//             -----------------
//         /|\|              XIN|-
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//            |                 |
//            |                 |
//            |         SDA/P2.7|-------> [I2C SLAVE SDA]
//            |         SCL/P2.6|-------> [I2C SLAVE SCL]
//
//  Note: Internal pull-ups are NOT used for SDA & SCL [DISABLED]
//
//  R. Wu
//  Texas Instruments Inc.
//  March 2008
//
// angepasst fr MSP430G2553
//	16.08.2012 DL9MWE
//******************************************************************************

#include "MSP430_SWI2C_Master.h"

void MSP430_SWI2CMST_init(void)
{
  PxSEL &= ~(SCL | SDA);                    // Set GPIO function
  PxDIR |= (SCL | SDA);                     // Set output direction
  PxOUT &= ~(SCL | SDA);                    // SCL & SDA = 0 => low when = outputs
  PxDIR &= ~(SCL | SDA);                    // set to input, SCL & SDA = 1
}

void MSP430_SWI2CMST_readBlock(unsigned char SlaveAddress,
                          unsigned int numBytes, void* RxData)
{
  unsigned char* temp;

  temp = (unsigned char *)RxData;           // Initialize array pointer
  MSP430_SWI2CMST_start();                  // Send Start condition
  unsigned int i;
  for (i = 0; i < numBytes; i++) {
    MSP430_SWI2CMST_txByte((SlaveAddress << 1) | BIT0); // [ADDR] + R/W bit = 1
    *(temp) = MSP430_SWI2CMST_rxByte();     // Read 8 bits of data
    MSP430_SWI2CMST_ack();					// send Master Acknowlegde
    temp++;                                 // Increment pointer to next element
  }
  MSP430_SWI2CMST_stop();                   // Send Stop condition
}

void MSP430_SWI2CMST_writeBlock(unsigned char SlaveAddress,
                           unsigned int numBytes, void* TxData)
{
   unsigned char *temp;

   temp = (unsigned char *)TxData;          // Initialize array pointer
   MSP430_SWI2CMST_start();                 // Send Start condition
   MSP430_SWI2CMST_txByte((SlaveAddress << 1) & ~BIT0); // [ADDR] + R/W bit = 0
   unsigned int i;
   for (i = 0; i < numBytes; i++) {
     MSP430_SWI2CMST_txByte(*(temp));       // Send data
     MSP430_SWI2CMST_ack();					// ack
     temp++;                                // Increment pointer to next element
   }
   MSP430_SWI2CMST_stop();                  // Send Stop condition
}


// Set I2C Start Condition
void MSP430_SWI2CMST_start(void)            // Set up start condition for I2C
{
  PxDIR |= SDA;                             // Set to output, data low [SDA = 0]
#ifdef DELAY_NOP
  _NOP();                                   // Quick delay
#endif
  PxDIR |= SCL;                             // Set to output, clock low [SCL = 0]
}

void MSP430_SWI2CMST_txByte(unsigned char data)
{
  unsigned char bits, temp;

  temp = data;
  bits = 0x08;                              // Load I2C bit counter
  while (bits != 0x00)                      // Loop until all bits are shifted
  {
    if (temp & BIT7)                        // Test data bit
      PxDIR &= ~SDA;                        // Set to input, SDA = 1 via pull-up
    else
      PxDIR |= SDA;                         // Set to output, data low [SDA = 0]
    PxDIR &= ~SCL;                          // Set to input, clock high [SCL = 1]
    temp = (temp << 1);                     // Shift bits 1 place to the left
    PxDIR |= SCL;                           // Set to output, clock low [SCL = 0]
    bits = (bits - 1);                      // Loop until 8 bits are sent
  }
  PxDIR &= ~SDA;                            // Set to input, Slave setzt SDA
}

void MSP430_SWI2CMST_ack(void)              // Set up for I2C acknowledge
{
   PxDIR &= ~SCL;                           // Set to input, SCL = 1 via pull-up
#ifdef DELAY_NOP
   _NOP();                                  //   "        "        "
#endif
   PxDIR |= SCL;                            // Set to output, clock low [SCL = 0]
}

void MSP430_SWI2CMST_nack(void)              // Send negative acknowledge
{
	PxDIR &= ~SDA;                            // Set to input, SDA = 1 via pull-up
#ifdef DELAY_NOP
	_NOP();                                  // delay to meet I2C spec
#endif
	PxDIR &= ~SCL;                           // Set to input, SCL = 1 via pull-up
#ifdef DELAY_NOP
	_NOP();                                  // delay to meet I2C spec
#endif
	PxDIR |= SCL;                            // Set to output, clock low [SCL = 0]
}

unsigned int MSP430_SWI2CSLV_ack(void)      // Get Slave ACK
{
  unsigned int error;
   PxDIR &= ~SCL;                           // Set to input, SCL = 1 via pull-up
#ifdef DELAY_NOP
   _NOP();                                  // delay to meet I2C spec
#endif
   if (PxIN & SDA)
	   error = 1;
   else
	   error = 0;
   PxDIR |= SCL;                            // Set to output, clock low [SCL = 0]
   return error;
}


// liest ein Byte via I2C ein
unsigned char MSP430_SWI2CMST_rxByte(void)  // Read 8 bits of I2C data
{
   unsigned char bits, temp = 0;            // I2C bit counter

   bits = 0x08;                             // Load I2C bit counter
   while (bits > 0)                         // Loop until all bits are read
   {
      PxDIR &= ~SCL;                        // Set to input, SCL = 1 via pull-up
      temp = (temp << 1);                   // Shift bits 1 place to the left
      if (PxIN & SDA)                       // Check digital input
        temp = (temp + 1);                  // If input is 'H' store a '1'
      PxDIR |= SCL;                         // Set to output, data low [SCL = 0]
      bits = (bits - 1);                    // Decrement I2C bit counter
   }
   return (temp);                           // Return 8-bit data byte
}


void MSP430_SWI2CMST_stop(void)             // Send I2C stop condition
{
  PxDIR |= SDA;                             // Set to output, data low [SDA = 0]
#ifdef DELAY_NOP
  _NOP();                                   // Quick delay
#endif
  PxDIR &= ~SCL;                            // Set to input, SCL = 1 via pull-up
#ifdef DELAY_NOP
  _NOP();                                   // Quick delay
#endif
  PxDIR &= ~SDA;                            // Set to input, SDA = 1 via pull-up
}

void MSP430_SWI2CMST_repeatedStart(void)    // Send I2C repeated start condition
{
	  PxDIR &= ~SDA;	// Set to input, SDA = 1 via pull-up
#ifdef DELAY_NOP
	  _NOP();			// Quick delay
#endif
	  PxDIR &= ~SCL;	// Set to input, SCL = 1 via pull-up
#ifdef DELAY_NOP
	  _NOP();			// Quick delay
#endif
	  PxDIR |= SDA;     // Set to output, data low [SDA = 0]
#ifdef DELAY_NOP
	  _NOP();			// Quick delay
#endif
	   PxDIR |= SCL;	// Set to output, clock low [SCL = 0]
}

// Ende
